/*
 * Copyright (c) 1993 Addison-Wesley Publishing Company.
 * All rights reserved.  This source code is provided
 * for its instructional value, and is not guaranteed
 * for any particular purpose.
 */
#include <sys/types.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <signal.h>

#define LISTSZ	20

struct list {
	struct list	*next;
	int			val;
} list[LISTSZ];
struct list *head;

void handler(int);
void printlist(int);

void
main(int argc, char *argv[])
{
	int i, n;

	if (argc == 2)
		n = atoi(argv[1]);
	else
		n = 0;

	/*
	 * Install a signal handler for SIGINT.
	 */
	sigset(SIGINT, handler);

	/*
	 * Build a linked list.
	 */
	for (i = LISTSZ-1; i >= 0; i--) {
		list[i].val = i+1;
		list[i].next = head;
		head = &list[i];
	}

	/*
	 * Print the list, generating SIGINT when we
	 * reach element "n".
	 */
	printlist(n);

	/*
	 * Print the list without generating signals.
	 */
	printlist(0);
	exit(0);
}

void
printlist(int n)
{
	struct list *p;

	/*
	 * Block SIGINT.
	 */
	sighold(SIGINT);

	/*
	 * Print the value of each element in the list.
	 */
	for (p = head; p != NULL; p = p->next) {
		printf("%d ", p->val);

		/*
		 * If we've reached the specified element,
		 * send ourselves SIGINT.
		 */
		if (p->val == n)
			kill(getpid(), SIGINT);
	}

	/*
	 * Unblock SIGINT.
	 */
	sigrelse(SIGINT);
	printf("\n");
}

void
handler(int sig)
{
	struct list *p, *op;

	/*
	 * Search the list and remove element 12.
	 */
	op = NULL;
	for (p = head; p != NULL; p = p->next) {
		if (p->val == 12) {
			if (op != NULL)
				op->next = p->next;
			else
				head = p->next;
			p->next = NULL;
			break;
		}
		op = p;
	}
}
