/*
 * Copyright (c) 1993 Addison-Wesley Publishing Company.
 * All rights reserved.  This source code is provided
 * for its instructional value, and is not guaranteed
 * for any particular purpose.
 */
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <stropts.h>

#define PIPEPATH	"/var/.authpipe"
#define ALLRD	(S_IRUSR|S_IRGRP|S_IROTH)
#define ALLWR	(S_IWUSR|S_IWGRP|S_IWOTH)
#define PIPEMODE	(ALLRD|ALLWR)

int pfd[2];

extern void fatal(const char *fmt, ...);

void
initialize()
{
	/*
	 * Create a pipe.
	 */
	if (pipe(pfd) < 0)
		fatal("cannot create pipe");

	/*
	 * Push CONNLD on one end to enable unique
	 * connections.
	 */
	if (ioctl(pfd[1], I_PUSH, "connld") < 0)
		fatal("cannot push CONNLD");

	/*
	 * Create a place to mount the pipe.
	 */
	close(creat(PIPEPATH, PIPEMODE));

	/*
	 * Mount the end of the pipe containing
	 * CONNLD on PIPEPATH.
	 */
	if (fattach(pfd[1], PIPEPATH) < 0)
		fatal("cannot attach pipe to file system");
}

void
serve()
{
	struct strrecvfd recv;
	struct strrecvfd conn;
	int okay;

	for (;;) {
		/*
		 * Get the file descriptor of the pipe
		 * connected to the local client process.
		 */
		if (ioctl(pfd[0], I_RECVFD, &conn) < 0)
			continue;

		/*
		 * Get the file descriptor to be authenticated.
		 */
		if (ioctl(conn.fd, I_RECVFD, &recv) < 0) {
			close(conn.fd);
			continue;
		}

		/*
		 * Authenticate the connection.
		 */
		okay = doauth(recv.fd);

		/*
		 * Send authenticated file descriptor back to
		 * the client process.
		 */
		if (okay)
			ioctl(conn.fd, I_SENDFD, recv.fd);

		/*
		 * Close the file descriptors we no longer need.
		 */
		close(conn.fd);
		close(recv.fd);
	}
}
