/*
 * Copyright (c) 1993 Addison-Wesley Publishing Company.
 * All rights reserved.  This source code is provided
 * for its instructional value, and is not guaranteed
 * for any particular purpose.
 */
#include <sys/types.h>
#include <rpc/types.h>
#include <rpc/xdr.h>
#include <stdio.h>
#include <stdlib.h>
#include <utmp.h>

extern void fatal(const char *, ...);

void
main(int argc, char *argv[])
{
	FILE *fp;
	XDR xh;
	struct utmp *up;
	int err = 0;

	if (argc != 3) {
		fprintf(stderr, "usage: mkut infile outfile\n");
		exit(1);
	}

	/*
	 * Open the utmp file.
	 */
	if (utmpname(argv[1]) != 1)
		fatal("mkut: input filename too long");

	/*
	 * Create the output file.
	 */
	if ((fp = fopen(argv[2], "w")) == NULL)
		fatal("mkut: cannot create %s", argv[2]);

	/*
	 * Create the XDR stream.
	 */
	xdrstdio_create(&xh, fp, XDR_ENCODE);

	/*
	 * Encode each utmp entry.
	 */
	while ((up = getutent()) != NULL) {
		if (xdr_vector(&xh, up->ut_user,
		  sizeof(up->ut_user), sizeof(char),
		  xdr_char) == FALSE) {
			err = 1;
			break;
		}
		if (xdr_vector(&xh, up->ut_id, sizeof(up->ut_id),
		  sizeof(char), xdr_char) == FALSE) {
			err = 2;
			break;
		}
		if (xdr_vector(&xh, up->ut_line,
		  sizeof(up->ut_line), sizeof(char),
		  xdr_char) == FALSE) {
			err = 3;
			break;
		}
		if (xdr_short(&xh, &up->ut_pid) == FALSE) {
			err = 4;
			break;
		}
		if (xdr_short(&xh, &up->ut_type) == FALSE) {
			err = 5;
			break;
		}
		if (xdr_short(&xh, &up->ut_exit.e_termination) ==
		  FALSE) {
			err = 6;
			break;
		}
		if (xdr_short(&xh, &up->ut_exit.e_exit) ==
		  FALSE) {
			err = 7;
			break;
		}
		if (xdr_long(&xh, &up->ut_time) == FALSE) {
			err = 8;
			break;
		}
	}
	if (err != 0)
		fatal("mkut: XDR error %d converting data", err);
	exit(0);
}
